package cn.estsh.i3plus.core.api.iservice.busi;

import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskTime;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskTimeExpression;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogTaskTime;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * @Description : 定时任务业务接口
 * @Reference   :
 * @Author      : wei.peng
 * @Date        : 2018-10-22 16:58:43.968
 * @Modify      :
 **/
public interface ISysTaskTimeService {

    /**
     *  新增定时任务
     * @param taskTime
     * @return
     */
    @ApiOperation(value = "新增定时任务")
    SysTaskTime insertSysTaskTime(SysTaskTime taskTime, SysTaskTimeExpression taskTimeExpression);

    /**
     * 修改定时任务
     * @param taskTime
     */
    @ApiOperation(value = "修改定时任务")
    void updateSysTaskTime(SysTaskTime taskTime, SysTaskTimeExpression taskTimeExpression);

    /**
     * 修改任务状态
     * @param id        任务ID
     * @param status    任务状态
     * @param user      当前会话用户
     */
    @ApiOperation(value = "修改任务状态")
    void updateSysTaskTimeStatus(Long id, int status, SessionUser user);

    /**
     * 根据根据id删除数据
     * @param id
     */
    @ApiOperation(value = "根据根据id删除数据")
    void deleteSysTaskTimeById(Long id);

    /**
     * 查询所有数据
     * @return
     */
    @ApiOperation(value = "查询所有数据")
    List<SysTaskTime> listSysTaskTime();

    /**
     * 根据id 数据
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id 数据")
    SysTaskTime getSysTaskTimeById(Long id);

    /**
     * 定时任务复制查询，分页，排序
     * @param taskTime
     * @param pager
     * @return
     */
    @ApiOperation(value = "定时任务复制查询，分页，排序")
    ListPager querySysTaskTimeByPager(SysTaskTime taskTime, Pager pager);

    /**
     * 根据id批量删除定时任务
     * @param ids
     */
    @ApiOperation(value = "根据id批量删除定时任务")
    void deleteSysTaskTimeByIds(Long[] ids);

    /**
     * 批量修改定时任务状态
     * @param ids
     *
     */
    @ApiOperation(value = "批量修改定时任务状态")
    void updateSysTaskTimeStatusByIds(Long[] ids, Integer status);

    /**
     * 修改最后时间
     * @param lastDate
     * @param name
     * @param groupName
     */
    @ApiOperation(value = "修改最后时间")
    void doSysTaskTimeLastDateByNameAndGroupName(String lastDate,String name,String groupName);

    /**
     * 立即执行
     * @param id
     */
    @ApiOperation(value = "立即执行")
    void executeSysTaskTimeById(Long id);

    /**
     * 根据定时任务名称与组名称查询
     * @param name
     * @param groupName
     * @return
     */
    @ApiOperation(value = "查询定时任务",notes = "根据定时任务名称与组名称查询")
    SysTaskTime getSysTaskTimeByNameAndGroup(String name,String groupName);

    /**
     * 添加定时任务执行日志
     * @param sysLogTaskTime
     */
    @ApiOperation(value = "添加定时任务执行日志",notes = "添加定时任务执行日志")
    void logInsertSysLogTaskTime(SysLogTaskTime sysLogTaskTime);

    /**
     * 修改定时任务信息，不会更新quartz数据库
     * @param sysTaskTime
     */
    @ApiOperation(value = "修改定时任务信息",notes = "修改定时任务信息，不会更新quartz数据库")
    void updateSysTaskTime(SysTaskTime sysTaskTime);
}
