package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysDashboardService;
import cn.estsh.i3plus.platform.common.tool.SerializeTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.web.session.mgt.DefaultWebSessionManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description : Dashboard服务
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-12-10 17:04
 * @Modify:
 **/
@RestController
@Api(description = "Dashboard服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/dashboard")
public class SysDashboardController extends CoreBaseController {

    @Autowired
    private ISysDashboardService sysDashboardService;

    @Resource(name = "imppSessionManager")
    private DefaultWebSessionManager webSessionManager;

    @Resource(name = "redisCore")
    private ImppRedis redisCore;

    @GetMapping("/get-basic")
    @ApiOperation(value = "获取基础数据", notes = "获取基础数据，用户，部门，组织等信息")
    public ResultBean getBasicInfo() {
        try {
            // 保持时间,key统一
            Date nowDate = new Date();
            String redisKey = PlatformConstWords.REDIS_DASHBOARD_BASIC_INFO + "_" + nowDate.getTime();

            Map<Object, Object> dashboardData = redisCore.getHashMap(redisKey);
            if (dashboardData == null || dashboardData.size() == 0) {
                dashboardData = new HashMap<>();
                dashboardData.put("orgNum", sysDashboardService.getSysOrganizeCount());
                dashboardData.put("deptNum", sysDashboardService.getSysDepartmentCount());
                dashboardData.put("onlineUser", webSessionManager.getSessionDAO().getActiveSessions().size());
                dashboardData.put("userLoginLog", sysDashboardService.findSysUserLoginLog(10));


                redisCore.putHashMap(redisKey, dashboardData, 180);
            }
            return ResultBean.success("查询成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultMap(dashboardData);
        } catch (
                ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/get-log")
    @ApiOperation(value = "查询日志", notes = "最近操作，异常日志")
    public ResultBean getLogInfo() {
        try {
            // 保持时间,key统一
            Date nowDate = new Date();
            String redisKey = PlatformConstWords.REDIS_DASHBOARD_AVG_RESP_TIME + "_" + nowDate.getTime();

            Map<Object, Object> dashboardData = redisCore.getHashMap(redisKey);
            if (dashboardData == null || dashboardData.size() == 0) {
                dashboardData = new HashMap<>();
                dashboardData.put("logOperate", sysDashboardService.findNewSysLogOperateSize(10));
                dashboardData.put("logException", sysDashboardService.findNewSysLogExceptionSize(10));

                redisCore.putHashMap(redisKey, dashboardData, 180);
            }

            return ResultBean.success("查询成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultMap(dashboardData);
        } catch (
                ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/get-avg-response-time")
    @ApiOperation(value = "获取平均响应时间", notes = "获取平均响应时间")
    public ResultBean getAvgResponseTime() {
        try {
            // 保持时间,key统一
            Date nowDate = new Date();
            String redisKey = PlatformConstWords.REDIS_DASHBOARD_AVG_RESP_TIME + "_" + nowDate.getTime();

            Map<Object, Object> dashboardData = redisCore.getHashMap(redisKey);
            if (dashboardData == null || dashboardData.size() == 0) {
                // 获取前5分钟时间
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

                dashboardData = sysDashboardService.querySysLogSystemAvgExecuteTime(
                        sdf.format(TimeTool.timeCalc(nowDate, Calendar.MINUTE, -5)),sdf.format(nowDate));

                redisCore.putHashMap(redisKey, dashboardData, 180);
            }
            return ResultBean.success("查询成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultMap(dashboardData);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        }
    }

    @GetMapping(value = "/get-level-log-num")
    @ApiOperation(value = "获取各等级日志数量", notes = "获取各等级日志数量")
    public ResultBean getLevelLogNum() {
        try {
            // 保持时间,key统一
            Date nowDate = new Date();
            String redisKey = PlatformConstWords.REDIS_DASHBOARD_LOG_NUM + "_" + nowDate.getTime();

            Map<Object, Object> dashboardData = redisCore.getHashMap(redisKey);
            if (dashboardData == null || dashboardData.size() == 0) {
                // 获取前10分钟时间，日志数量
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

                dashboardData = sysDashboardService.querySysLogSystemByLevel(
                        sdf.format(TimeTool.timeCalc(new Date(), Calendar.MINUTE, -10)), sdf.format(nowDate));

                redisCore.putHashMap(redisKey, dashboardData, 180);
            }
            return ResultBean.success("查询成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultMap(dashboardData);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/get-server-pressure")
    public ResultBean getServerPressure() {
        try {
            // 保持时间,key统一
            Date nowDate = new Date();
            String redisKey = PlatformConstWords.REDIS_DASHBOARD_SERVER_PRESSURE + "_" + nowDate.getTime();

            Map<Object, Object> dashboardData = redisCore.getHashMap(redisKey);
            if (dashboardData == null || dashboardData.size() == 0) {
                dashboardData = new HashMap<>();
                //获取上一分钟,请求来源数据
                SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmm");

                dashboardData.put("requestSource", redisCore.getHashMap(
                        CommonConstWords.REDIS_DASHBOARD_MINUTE_SOURCE + "_" + sdf.format(TimeTool.timeCalc(nowDate, Calendar.MINUTE, -1))));

                //获取上一秒，服务器压力数据
                sdf = new SimpleDateFormat("yyyyMMddHHmmss");

                dashboardData.put("serverPressure", redisCore.getObject(
                        CommonConstWords.REDIS_DASHBOARD_SECOND_PRESSURE + "_" + sdf.format(TimeTool.timeCalc(nowDate, Calendar.SECOND, -1))));

                redisCore.putHashMap(redisKey, dashboardData, 180);
            }
            return ResultBean.success("查询成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultMap(dashboardData);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
