package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.core.api.iservice.busi.ISysMessageService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.core.apiservice.websocket.MessageWebSocket;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysMessage;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserMessage;
import cn.estsh.i3plus.pojo.platform.bean.SysUser;
import com.alibaba.fastjson.JSON;
import com.rabbitmq.client.Channel;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.List;

/**
 * @Description : 站内信队列处理
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-11-15 22:35
 * @Modify:
 **/
@Component
public class MessageLetterQueueReceiver {
    private static final Logger LOGGER = LoggerFactory.getLogger(MessageLetterQueueReceiver.class);

    @Autowired
    ISysMessageService sysMessageService;
    @Autowired
    ISysUserService sysUserService;

    /**
     * 站内信处理队列
     * @param msg
     * @param channel
     * @param message
     * 发送：rabbitTemplate.convertAndSend(I3CoreQueueConfig.IMPP_MESSAGE_QUEUE, new SysMessage(....));
     */
    @RabbitListener(queues = PlatformConstWords.IMPP_MESSAGE_LETTER_QUEUE)
    public void processImppMessage(SysMessage msg, Channel channel, Message message) {
        try {
            LOGGER.info("【MQ-IMPP_MESSAGE_LETTER_QUEUE】数据接收成功：{}",msg);

            msg = sysMessageService.insertSysMessage(msg);

            // 收件人信息
            String[] messageReceiver = msg.getMessageReceiversId().split(",");
            String[] receiverName = new String[messageReceiver.length];

            SysRefUserMessage refUserMessage;
            SysUser sysUser;
            List userMessage;

            for (int i = 0; i < messageReceiver.length; i++) {
                sysUser = sysUserService.getSysUserById(Long.parseLong(messageReceiver[i]));
                receiverName[i] = sysUser.getUserName();

                refUserMessage = new SysRefUserMessage();
                refUserMessage.setMessageId(msg.getId());
                refUserMessage.setMessageTitleRdd(msg.getMessageTitle());
                refUserMessage.setMessageTypeRdd(msg.getMessageType());
                refUserMessage.setMessageSenderNameRdd(msg.getMessageSenderNameRdd());
                refUserMessage.setReceiverId(sysUser.getUserInfoId());
                refUserMessage.setReceiverNameRdd(sysUser.getUserName());
                refUserMessage.setMessageStatus(ImppEnumUtil.MESSAGE_STATUS.UNREAD.getValue());
                refUserMessage.setReceiverTime(TimeTool.getNowTime(true));

                sysMessageService.insertSysRefUserMessage(refUserMessage);

                userMessage = sysMessageService.findSysRefUserMessageByUserIdAndStatus(sysUser.getUserInfoId(),
                        ImppEnumUtil.MESSAGE_STATUS.UNREAD.getValue());
                MessageWebSocket.sendMessage(sysUser.getUserInfoId(),
                        JSON.toJSONString(userMessage)
                );
            }

            msg.setMessageReceiversNameRdd(StringUtils.join(receiverName, ","));
            sysMessageService.updateSysMessage(msg);


            //信息已处理
            channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
        } catch (IOException e) {
            LOGGER.error("【MQ-IMPP_MESSAGE_LETTER_QUEUE】处理出错：{}",e.getMessage(),e);
            //丢弃这条消息
            try {
                // 未成功处理，重新发送
                channel.basicNack(message.getMessageProperties().getDeliveryTag(),false,true);
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
    }
}
